from __future__ import absolute_import, unicode_literals

from jsonfield import JSONField

from . import fields
from .conf import settings
from .widgets import SirTrevorWidget
from .render import render_json


def default():
    return {'data': []}


class SirTrevorField(JSONField):

    def __init__(self, blocks=None, **kwargs):
        kwargs.setdefault('default', default)
        super(SirTrevorField, self).__init__(**kwargs)
        self.blocks = blocks

    def formfield(self, **kwargs):
        blocks = self.get_blocks()
        options = {
            'blocks': blocks,
            'form_class': fields.SirTrevorField,
            'widget': SirTrevorWidget(blocks=blocks),
        }
        options.update(**kwargs)
        out = super(SirTrevorField, self).formfield(**options)
        return out

    def get_blocks(self):
        blocks = self.blocks
        if blocks is None:
            blocks = settings.get_default_blocks()
        return list(b() if isinstance(b, type) else b for b in blocks)

    def contribute_to_class(self, cls, name):
        super(SirTrevorField, self).contribute_to_class(cls, name)
        render = lambda model: render_json(getattr(model, self.name), self.get_blocks())
        setattr(cls, 'get_{0}_html'.format(self.name), render)

    def deconstruct(self):
        name, path, args, kwargs = super(SirTrevorField, self).deconstruct()
        kwargs['blocks'] = self.blocks
        return name, path, args, kwargs

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^sirdjango\.models\.SirTrevorField"])
except ImportError:
    pass



from __future__ import absolute_import, unicode_literals

import re
import six

from django.core.exceptions import ValidationError
from django.forms.widgets import MediaDefiningClass

from .conf import settings, import_string


def underscored(name):
    """
    A port of the SirTrevor.js `underscored` function. This transforms block
    names in to types for serialisation.
    """
    name = name.strip()
    name = re.sub(r'([a-z\d])([A-Z]+)', r'\1_\2', name)
    name = re.sub(r'[-\s]+', '_', name)
    return name.lower()


class BaseBlock(six.with_metaclass(MediaDefiningClass, object)):

    name = None

    @property
    def block_type(self):
        return underscored(self.name)

    def render_json(self, json):
        raise NotImplementedError

    def clean(self, data):
        """
        Clean incoming JSON data.

        If the data are not valid, raises a ValidationError
        """
        return data


class TextBlock(BaseBlock):
    name = 'Text'

    def clean(self, data):
        data = super(TextBlock, self).clean(data)
        if set(['text']) - set(data.keys()):
            raise ValidationError('Invalid data for {0} block'.format(self.name))
        return data

    def render_json(self, data):
        render_markdown = import_string(settings.SIRDJANGO_RENDER_MARKDOWN)
        return render_markdown(data['text'])


class VideoBlock(BaseBlock):
    name = 'Video'

    protocol = ''
    providers = {
        'vimeo': {
            'html': r'<iframe src="{protocol}//player.vimeo.com/video/{remote_id}?title=0&byline=0" width="580" height="320" frameborder="0"></iframe>',
            'remote_id': re.compile(r'^\d+$'),
        },
        'youtube': {
            'html': r'<iframe src="{protocol}//www.youtube.com/embed/{remote_id}" width="580" height="320" frameborder="0" allowfullscreen></iframe>',
            'remote_id': re.compile(r'^[a-zA-Z0-9_-]{10,12}$'),
        }
    }

    def clean(self, data):
        data = super(VideoBlock, self).clean(data)
        if set(['source', 'remote_id']) - set(data.keys()):
            raise ValidationError('Invalid data for {0} block'.format(self.name))
        if data['source'] not in self.providers:
            raise ValidationError('Invalid video provider')
        provider = self.providers[data['source']]
        if provider['remote_id'].match(data['remote_id']) is None:
            raise ValidationError('Invalid video ID')
        return data

    def render_json(self, data):
        provider = self.providers[data['source']]
        return provider['html'].format(protocol=self.protocol,
                                       remote_id=data['remote_id'])



from django.conf import settings as base_settings

try:
    from django.utils.module_loading import import_string
except ImportError:
    from django.utils.module_loading import import_by_path as import_string


class Settings(object):

    SIRDJANGO_RENDER_MARKDOWN = 'sirdjango.markdown.render'
    SIRDJANGO_DEFAULT_BLOCKS = [
        'sirdjango.blocks.TextBlock',
        'sirdjango.blocks.VideoBlock'
    ]
    SIRDJANGO_MEDIA = {
        'js': [
            'sirdjango/js/jquery.js',
            'sirdjango/js/underscore.js',
            'sirdjango/js/eventable.js',
            'sirdjango/js/sir-trevor.js',
        ],
        'css': {'screen': [
            'sirdjango/css/sir-trevor.css',
            'sirdjango/css/sir-trevor-icons.css',
        ]},
    }

    def get_default_blocks(self):
        for block_string in self.SIRDJANGO_DEFAULT_BLOCKS:
            block_class = import_string(block_string)
            yield block_class()

    def __getattribute__(self, name):
        if hasattr(base_settings, name):
            return getattr(base_settings, name)
        return super(Settings, self).__getattribute__(name)


settings = Settings()



from __future__ import absolute_import, unicode_literals

import json

from django.core.exceptions import ValidationError
from django.forms import fields

from .conf import settings
from .widgets import SirTrevorWidget


class SirTrevorField(fields.Field):
    widget = SirTrevorWidget

    def __init__(self, blocks=None, **kwargs):
        super(SirTrevorField, self).__init__(**kwargs)
        self.blocks = blocks

    def to_python(self, value):
        try:
            value = json.loads(value)
        except ValueError:
            raise ValidationError("Invalid JSON data")
        return self.process_data(value)

    def get_blocks(self):
        blocks = self.blocks
        if blocks is None:
            blocks = settings.get_default_blocks()
        return list(b() if isinstance(b, type) else b for b in blocks)

    def process_data(self, data):
        if not isinstance(data, dict) or set(data.keys()) != set(['data']):
            raise ValidationError("Invalid JSON data")

        raw_blocks = data['data']
        if not isinstance(raw_blocks, list):
            raise ValidationError("Invalid JSON data")

        return {'data': list(self.process_blocks(raw_blocks))}

    def process_blocks(self, raw_blocks):
        block_types = self.get_blocks()
        block_type_map = dict((b.block_type, b) for b in block_types)
        for raw_block in raw_blocks:
            if raw_block['type'] not in block_type_map:
                raise ValidationError("Unknown block type")

            block_type = block_type_map[raw_block['type']]
            raw_block_data = raw_block['data']
            yield {
                'type': block_type.block_type,
                'data': block_type.clean(raw_block_data)
            }



from __future__ import absolute_import, print_function, unicode_literals

import json

from django.forms import widgets
from django.forms.widgets import flatatt
from django.utils.html import mark_safe, format_html

from .conf import settings


class SirTrevorWidget(widgets.Textarea):

    script_template = '<script>new SirTrevor.Editor({options});</script>'

    def __init__(self, blocks=None, extra_options={}, **kwargs):
        super(SirTrevorWidget, self).__init__(**kwargs)
        self.blocks = list(blocks)
        self.extra_options = extra_options

    @property
    def media(self):
        media = widgets.Media(**settings.SIRDJANGO_MEDIA)
        for block in self.blocks:
            media += block.media
        return media

    def render(self, name, value, attrs=None):
        if isinstance(value, (dict, list)):
            value = json.dumps(value)

        final_attrs = self.build_attrs(attrs, name=name)
        textarea = format_html('<textarea{0}>{1}</textarea>',
                               flatatt(final_attrs), value)

        id_ = final_attrs.get('id', None)
        if id_ is None:
            return textarea

        return mark_safe(textarea + self.script_template.format(
            options=self.get_options('#' + id_)))

    def get_options(self, element_selector):
        options = {
            'blockTypes': [block.name for block in self.blocks],
        }
        options.update(self.extra_options)
        # Unforunately, the 'el' option is not valid JSON, but the rest of the
        # options are. Ignore the handcrafted JSON, you saw nothing!
        return '{{el:$({element_selector}), {options}}}'.format(
            element_selector=json.dumps(element_selector),
            options=json.dumps(options)[1:-1])



def render_json(data, blocks):
    if not isinstance(data, dict):
        raise ValueError('Invalid block data')

    raw_blocks = data.get('data', None)
    if not isinstance(raw_blocks, list):
        raise ValueError('Invalid block data')

    block_map = dict((b.block_type, b) for b in blocks)
    return ''.join(render_block(raw_block, block_map) for raw_block in data['data'])


def render_block(raw_block, block_map):
    block = block_map[raw_block['type']]
    return block.render_json(raw_block['data'])



#!/usr/bin/env python
"""
Install sirdjango using setuptools
"""

from sirdjango import __version__

with open('README.rst', 'r') as f:
    readme = f.read()

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

setup(
    name='sirdjango',
    version=__version__,
    description='Sir Trevor integration with Django',
    long_description=readme,
    author='Tim Heap',
    author_email='tim@takeflight.com.au',
    url='https://bitbucket.org/takeflight/sirdjango',

    install_requires=[
        'django>=1.6',
        'django-jsonfield',
        'markdown',
    ],
    zip_safe=False,
    license='BSD License',

    packages=find_packages(),

    include_package_data=True,
    package_data={},

    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Framework :: Django',
        'License :: OSI Approved :: BSD License',
    ],
)
